'
'    StopWatch class for VB.NET
'		Version: 1.0
'
'
'    Copyright  2002, The KPD-Team
'    All rights reserved.
'    http://www.mentalis.org/
'
'  Redistribution and use in source and binary forms, with or without
'  modification, are permitted provided that the following conditions
'  are met:
'
'    - Redistributions of source code must retain the above copyright
'       notice, this list of conditions and the following disclaimer. 
'
'    - Neither the name of the KPD-Team, nor the names of its contributors
'       may be used to endorse or promote products derived from this
'       software without specific prior written permission. 
'
'  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
'  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
'  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
'  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
'  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
'  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
'  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
'  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
'  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
'  STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
'  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
'  OF THE POSSIBILITY OF SUCH DAMAGE.
'

Imports System
Imports System.Runtime.InteropServices

Namespace Org.Mentalis.Utilities
	'/// <summary>
	'/// Represents a high-resolution stopwatch. It can be used to measure very small intervals of time.
	'/// </summary>
	Public NotInheritable Class StopWatch
		'/// <summary>
		'/// The QueryPerformanceCounter function retrieves the current value of the high-resolution performance counter.
		'/// </summary>
		'/// <param name="x">Pointer to a variable that receives the current performance-counter value, in counts.</param>
		'/// <returns>If the function succeeds, the return value is nonzero.</returns>
		Private Declare Ansi Function QueryPerformanceCounter Lib "kernel32.dll" (ByRef x As Long) As Integer
		'/// <summary>
		'/// The QueryPerformanceFrequency function retrieves the frequency of the high-resolution performance counter, if one exists. The frequency cannot change while the system is running.
		'/// </summary>
		'/// <param name="x">Pointer to a variable that receives the current performance-counter frequency, in counts per second. If the installed hardware does not support a high-resolution performance counter, this parameter can be zero.</param>
		'/// <returns>If the installed hardware supports a high-resolution performance counter, the return value is nonzero.</returns>
		Private Declare Ansi Function QueryPerformanceFrequency Lib "kernel32.dll" (ByRef x As Long) As Integer
		'/// <summary>
		'/// Initializes a new instance of the StopWatch class.
		'/// </summary>
		'/// <exception cref="NotSupportedException">The system does not have a high-resolution performance counter.</exception>
		Public Sub New()
			Frequency = GetFrequency()
			Reset()
		End Sub
		'/// <summary>
		'/// Resets the stopwatch. This method should be called when you start measuring.
		'/// </summary>
		'/// <exception cref="NotSupportedException">The system does not have a high-resolution performance counter.</exception>
		Public Sub Reset()
			StartTime = GetValue()
		End Sub
		'/// <summary>
		'/// Returns the time that has passed since the Reset() method was called.
		'/// </summary>
		'/// <remarks>The time is returned in tenths-of-a-millisecond. If the Peek method returns '10000', it means the interval took exactely one second.</remarks>
		'/// <returns>A long that contains the time that has passed since the Reset() method was called.</returns>
		'/// <exception cref="NotSupportedException">The system does not have a high-resolution performance counter.</exception>
		Public Function Peek() As Long
			Return CType((((GetValue() - StartTime) / CType(Frequency, Double)) * 10000), Long)
		End Function
		'/// <summary>
		'/// Retrieves the current value of the high-resolution performance counter.
		'/// </summary>
		'/// <exception cref="NotSupportedException">The system does not have a high-resolution performance counter.</exception>
		'/// <returns>A long that contains the current performance-counter value, in counts.</returns>
		Private Function GetValue() As Long
			Dim ret As Long = 0
			If QueryPerformanceCounter(ret) = 0 Then Throw New NotSupportedException("Error while querying the high-resolution performance counter.")
			Return ret
		End Function
		'/// <summary>
		'/// Retrieves the frequency of the high-resolution performance counter, if one exists. The frequency cannot change while the system is running.
		'/// </summary>
		'/// <exception cref="NotSupportedException">The system does not have a high-resolution performance counter.</exception>
		'/// <returns>A long that contains the current performance-counter frequency, in counts per second.</returns>
		Private Function GetFrequency() As Long
			Dim ret As Long = 0
			If QueryPerformanceFrequency(ret) = 0 Then Throw New NotSupportedException("Error while querying the performance counter frequency.")
			Return ret
		End Function
		'/// <summary>
		'/// Gets or sets the start time.
		'/// </summary>
		'/// <value>A long that holds the start time.</value>
		Private Property StartTime() As Long
			Get
				Return m_StartTime
			End Get
			Set(ByVal Value As Long)
				m_StartTime = Value
			End Set
		End Property
		'/// <summary>
		'/// Gets or sets the frequency of the high-resolution performance counter.
		'/// </summary>
		'/// <value>A long that holds the frequency of the high-resolution performance counter.</value>
		Private Property Frequency() As Long
			Get
				Return m_Frequency
			End Get
			Set(ByVal Value As Long)
				m_Frequency = Value
			End Set
		End Property
		' private variables
		'/// <summary>Holds the value of the StartTime property.</summary>
		Private m_StartTime As Long
		'/// <summary>Holds the value of the Frequency property.</summary>
		Private m_Frequency As Long
	End Class
End Namespace